@@ -41,7 +41,7 @@ be one of the following:
    processed by the :ref:`Watcher Applier <watcher_applier_definition>`
 -  **SUCCEEDED** : the :ref:`Action <action_definition>` has been executed
    successfully
--  **FAILED** : an error occured while trying to execute the
+-  **FAILED** : an error occurred while trying to execute the
    :ref:`Action <action_definition>`
 -  **DELETED** : the :ref:`Action <action_definition>` is still stored in the
    :ref:`Watcher database <watcher_database_definition>` but is not returned
@@ -88,7 +88,6 @@ class Action(base.APIBase):
     between the internal object model and the API representation of a action.
     """
     _action_plan_uuid = None
-    _next_uuid = None
 
     def _get_action_plan_uuid(self):
         return self._action_plan_uuid
@@ -105,22 +104,6 @@ class Action(base.APIBase):
             except exception.ActionPlanNotFound:
                 self._action_plan_uuid = None
 
-    def _get_next_uuid(self):
-        return self._next_uuid
-
-    def _set_next_uuid(self, value):
-        if value == wtypes.Unset:
-            self._next_uuid = wtypes.Unset
-        elif value and self._next_uuid != value:
-            try:
-                action_next = objects.Action.get(
-                    pecan.request.context, value)
-                self._next_uuid = action_next.uuid
-                self.next = action_next.id
-            except exception.ActionNotFound:
-                self.action_next_uuid = None
-                # raise e
-
     uuid = wtypes.wsattr(types.uuid, readonly=True)
     """Unique UUID for this action"""
 
@@ -135,13 +118,14 @@ class Action(base.APIBase):
     action_type = wtypes.text
     """Action type"""
 
+    description = wtypes.text
+    """Action description"""
+
     input_parameters = types.jsontype
     """One or more key/value pairs """
 
-    next_uuid = wsme.wsproperty(types.uuid, _get_next_uuid,
-                                _set_next_uuid,
-                                mandatory=True)
-    """This next action UUID"""
+    parents = wtypes.wsattr(types.jsontype, readonly=True)
+    """UUIDs of parent actions"""
 
     links = wsme.wsattr([link.Link], readonly=True)
     """A list containing a self link and associated action links"""
@@ -152,7 +136,6 @@ class Action(base.APIBase):
         self.fields = []
         fields = list(objects.Action.fields)
         fields.append('action_plan_uuid')
-        fields.append('next_uuid')
         for field in fields:
             # Skip fields we do not expose.
             if not hasattr(self, field):
@@ -161,17 +144,16 @@ class Action(base.APIBase):
             setattr(self, field, kwargs.get(field, wtypes.Unset))
 
         self.fields.append('action_plan_id')
+        self.fields.append('description')
         setattr(self, 'action_plan_uuid', kwargs.get('action_plan_id',
                 wtypes.Unset))
-        setattr(self, 'next_uuid', kwargs.get('next',
-                wtypes.Unset))
 
     @staticmethod
     def _convert_with_links(action, url, expand=True):
         if not expand:
-            action.unset_fields_except(['uuid', 'state', 'next', 'next_uuid',
-                                        'action_plan_uuid', 'action_plan_id',
-                                        'action_type'])
+            action.unset_fields_except(['uuid', 'state', 'action_plan_uuid',
+                                        'action_plan_id', 'action_type',
+                                        'parents'])
 
         action.links = [link.Link.make_link('self', url,
                                             'actions', action.uuid),
@@ -184,6 +166,14 @@ class Action(base.APIBase):
     @classmethod
     def convert_with_links(cls, action, expand=True):
         action = Action(**action.as_dict())
+        try:
+            obj_action_desc = objects.ActionDescription.get_by_type(
+                pecan.request.context, action.action_type)
+            description = obj_action_desc.description
+        except exception.ActionDescriptionNotFound:
+            description = ""
+        setattr(action, 'description', description)
+
         return cls._convert_with_links(action, pecan.request.host_url, expand)
 
     @classmethod
@@ -193,9 +183,9 @@ class Action(base.APIBase):
                      state='PENDING',
                      created_at=datetime.datetime.utcnow(),
                      deleted_at=None,
-                     updated_at=datetime.datetime.utcnow())
+                     updated_at=datetime.datetime.utcnow(),
+                     parents=[])
         sample._action_plan_uuid = '7ae81bb3-dec3-4289-8d6c-da80bd8001ae'
-        sample._next_uuid = '7ae81bb3-dec3-4289-8d6c-da80bd8001ae'
         return cls._convert_with_links(sample, 'http://localhost:9322', expand)
 
 
@@ -215,17 +205,6 @@ class ActionCollection(collection.Collection):
         collection = ActionCollection()
         collection.actions = [Action.convert_with_links(p, expand)
                               for p in actions]
-
-        if 'sort_key' in kwargs:
-            reverse = False
-            if kwargs['sort_key'] == 'next_uuid':
-                if 'sort_dir' in kwargs:
-                    reverse = True if kwargs['sort_dir'] == 'desc' else False
-                collection.actions = sorted(
-                    collection.actions,
-                    key=lambda action: action.next_uuid or '',
-                    reverse=reverse)
-
         collection.next = collection.get_next(limit, url=url, **kwargs)
         return collection
 
@@ -253,6 +232,10 @@ class ActionsController(rest.RestController):
                                 sort_key, sort_dir, expand=False,
                                 resource_url=None,
                                 action_plan_uuid=None, audit_uuid=None):
+        additional_fields = ['action_plan_uuid']
+
+        api_utils.validate_sort_key(sort_key, list(objects.Action.fields) +
+                                    additional_fields)
         limit = api_utils.validate_limit(limit)
         api_utils.validate_sort_dir(sort_dir)
 
@@ -268,10 +251,10 @@ class ActionsController(rest.RestController):
         if audit_uuid:
             filters['audit_uuid'] = audit_uuid
 
-        if sort_key == 'next_uuid':
-            sort_db_key = None
-        else:
-            sort_db_key = sort_key
+        need_api_sort = api_utils.check_need_api_sort(sort_key,
+                                                      additional_fields)
+        sort_db_key = (sort_key if not need_api_sort
+                       else None)
 
         actions = objects.Action.list(pecan.request.context,
                                       limit,
@@ -279,11 +262,15 @@ class ActionsController(rest.RestController):
                                       sort_dir=sort_dir,
                                       filters=filters)
 
-        return ActionCollection.convert_with_links(actions, limit,
-                                                   url=resource_url,
-                                                   expand=expand,
-                                                   sort_key=sort_key,
-                                                   sort_dir=sort_dir)
+        actions_collection = ActionCollection.convert_with_links(
+            actions, limit, url=resource_url, expand=expand,
+            sort_key=sort_key, sort_dir=sort_dir)
+
+        if need_api_sort:
+            api_utils.make_api_sort(actions_collection.actions,
+                                    sort_key, sort_dir)
+
+        return actions_collection
 
     @wsme_pecan.wsexpose(ActionCollection, types.uuid, int,
                          wtypes.text, wtypes.text, types.uuid,
@@ -365,7 +352,7 @@ class ActionsController(rest.RestController):
 
     @wsme_pecan.wsexpose(Action, body=Action, status_code=201)
     def post(self, action):
-        """Create a new action.
+        """Create a new action(forbidden).
 
         :param action: a action within the request body.
         """
@@ -379,7 +366,7 @@ class ActionsController(rest.RestController):
         action_dict = action.as_dict()
         context = pecan.request.context
         new_action = objects.Action(context, **action_dict)
-        new_action.create(context)
+        new_action.create()
 
         # Set the HTTP Location Header
         pecan.response.location = link.build_url('actions', new_action.uuid)
@@ -388,7 +375,7 @@ class ActionsController(rest.RestController):
     @wsme.validate(types.uuid, [ActionPatchType])
     @wsme_pecan.wsexpose(Action, types.uuid, body=[ActionPatchType])
     def patch(self, action_uuid, patch):
-        """Update an existing action.
+        """Update an existing action(forbidden).
 
         :param action_uuid: UUID of a action.
         :param patch: a json PATCH document to apply to this action.
@@ -425,7 +412,7 @@ class ActionsController(rest.RestController):
 
     @wsme_pecan.wsexpose(None, types.uuid, status_code=204)
     def delete(self, action_uuid):
-        """Delete a action.
+        """Delete a action(forbidden).
 
         :param action_uuid: UUID of a action.
         """
