@@ -63,7 +63,7 @@ class ContextHook(hooks.PecanHook):
         auth_url = headers.get('X-Auth-Url')
         if auth_url is None:
             importutils.import_module('keystonemiddleware.auth_token')
-            auth_url = cfg.CONF.keystone_authtoken.auth_uri
+            auth_url = cfg.CONF.keystone_authtoken.www_authenticate_uri
 
         state.request.context = context.make_context(
             auth_token=auth_token,
@@ -114,6 +114,6 @@ class NoExceptionTracebackHook(hooks.PecanHook):
             faultstring = faultstring.split(traceback_marker, 1)[0]
             # Remove trailing newlines and spaces if any.
             json_body['faultstring'] = faultstring.rstrip()
-            # Replace the whole json. Cannot change original one beacause it's
+            # Replace the whole json. Cannot change original one because it's
             # generated on the fly.
             state.response.json = json_body
