@@ -16,31 +16,26 @@
 
 """Utilities and helper functions."""
 
+import datetime
+import random
+import re
+import string
+
+from croniter import croniter
+
 from jsonschema import validators
-from oslo_config import cfg
-from oslo_log import log as logging
+from oslo_log import log
+from oslo_utils import strutils
+from oslo_utils import uuidutils
+import six
+
 from watcher.common import exception
 
-import re
-import six
-import uuid
+from watcher import conf
 
+CONF = conf.CONF
 
-from watcher._i18n import _LW
-
-UTILS_OPTS = [
-    cfg.StrOpt('rootwrap_config',
-               default="/etc/watcher/rootwrap.conf",
-               help='Path to the rootwrap configuration file to use for '
-                    'running commands as root.'),
-    cfg.StrOpt('tempdir',
-               help='Explicitly specify the temporary working directory.'),
-]
-
-CONF = cfg.CONF
-CONF.register_opts(UTILS_OPTS)
-
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 
 class Struct(dict):
@@ -66,6 +61,20 @@ class Struct(dict):
             raise AttributeError(name)
 
 
+generate_uuid = uuidutils.generate_uuid
+is_uuid_like = uuidutils.is_uuid_like
+is_int_like = strutils.is_int_like
+
+
+def is_cron_like(value):
+    """Return True is submitted value is like cron syntax"""
+    try:
+        croniter(value, datetime.datetime.now())
+    except Exception as e:
+        raise exception.CronFormatIsInvalid(message=str(e))
+    return True
+
+
 def safe_rstrip(value, chars=None):
     """Removes trailing characters from a string if that does not make it empty
 
@@ -75,39 +84,14 @@ def safe_rstrip(value, chars=None):
 
     """
     if not isinstance(value, six.string_types):
-        LOG.warning(_LW(
+        LOG.warning(
             "Failed to remove trailing character. Returning original object."
-            "Supplied object is not a string: %s,"), value)
+            "Supplied object is not a string: %s,", value)
         return value
 
     return value.rstrip(chars) or value
 
 
-def generate_uuid():
-    return str(uuid.uuid4())
-
-
-def is_int_like(val):
-    """Check if a value looks like an int."""
-    try:
-        return str(int(val)) == str(val)
-    except Exception:
-        return False
-
-
-def is_uuid_like(val):
-    """Returns validation of a value as a UUID.
-
-    For our purposes, a UUID is a canonical form string:
-    aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa
-
-    """
-    try:
-        return str(uuid.UUID(val)) == val
-    except (TypeError, ValueError, AttributeError):
-        return False
-
-
 def is_hostname_safe(hostname):
     """Determine if the supplied hostname is RFC compliant.
 
@@ -133,17 +117,13 @@ def get_cls_import_path(cls):
     return module + '.' + cls.__name__
 
 
-def strtime(at):
-    return at.strftime("%Y-%m-%dT%H:%M:%S.%f")
-
-
 # Default value feedback extension as jsonschema doesn't support it
 def extend_with_default(validator_class):
     validate_properties = validator_class.VALIDATORS["properties"]
 
     def set_defaults(validator, properties, instance, schema):
         for prop, subschema in properties.items():
-            if "default" in subschema:
+            if "default" in subschema and instance is not None:
                 instance.setdefault(prop, subschema["default"])
 
             for error in validate_properties(
@@ -160,6 +140,9 @@ def extend_with_strict_schema(validator_class):
     validate_properties = validator_class.VALIDATORS["properties"]
 
     def strict_schema(validator, properties, instance, schema):
+        if instance is None:
+            return
+
         for para in instance.keys():
             if para not in properties.keys():
                 raise exception.AuditParameterNotAllowed(parameter=para)
@@ -173,3 +156,10 @@ def extend_with_strict_schema(validator_class):
 
 StrictDefaultValidatingDraft4Validator = extend_with_default(
     extend_with_strict_schema(validators.Draft4Validator))
+
+Draft4Validator = validators.Draft4Validator
+
+
+def random_string(n):
+    return ''.join([random.choice(
+        string.ascii_letters + string.digits) for i in range(n)])
