@@ -1,5 +1,6 @@
 # -*- encoding: utf-8 -*-
 # Copyright (c) 2015 b<>com
+# Copyright (c) 2016 Intel Corp
 #
 # Authors: Jean-Emile DARTOIS <jean-emile.dartois@b-com.com>
 #
@@ -17,20 +18,14 @@
 # limitations under the License.
 #
 
-from oslo_config import cfg
-
 from watcher.common import exception
-from watcher.common.messaging import notification_handler
 from watcher.common import service
+from watcher.common import service_manager
 from watcher.common import utils
-from watcher.decision_engine import manager
 
+from watcher import conf
 
-CONF = cfg.CONF
-
-CONF.register_group(manager.decision_engine_opt_group)
-CONF.register_opts(manager.WATCHER_DECISION_ENGINE_OPTS,
-                   manager.decision_engine_opt_group)
+CONF = conf.CONF
 
 
 class DecisionEngineAPI(service.Service):
@@ -42,21 +37,40 @@ class DecisionEngineAPI(service.Service):
         if not utils.is_uuid_like(audit_uuid):
             raise exception.InvalidUuidOrName(name=audit_uuid)
 
+        self.conductor_client.cast(
+            context, 'trigger_audit', audit_uuid=audit_uuid)
+
+    def get_strategy_info(self, context, strategy_name):
         return self.conductor_client.call(
-            context.to_dict(), 'trigger_audit', audit_uuid=audit_uuid)
+            context, 'get_strategy_info', strategy_name=strategy_name)
 
 
-class DecisionEngineAPIManager(object):
+class DecisionEngineAPIManager(service_manager.ServiceManager):
 
-    API_VERSION = '1.0'
+    @property
+    def service_name(self):
+        return None
 
-    conductor_endpoints = []
-    status_endpoints = [notification_handler.NotificationHandler]
-    notification_endpoints = []
-    notification_topics = []
+    @property
+    def api_version(self):
+        return '1.0'
 
-    def __init__(self):
-        self.publisher_id = CONF.watcher_decision_engine.publisher_id
-        self.conductor_topic = CONF.watcher_decision_engine.conductor_topic
-        self.status_topic = CONF.watcher_decision_engine.status_topic
-        self.api_version = self.API_VERSION
+    @property
+    def publisher_id(self):
+        return CONF.watcher_decision_engine.publisher_id
+
+    @property
+    def conductor_topic(self):
+        return CONF.watcher_decision_engine.conductor_topic
+
+    @property
+    def notification_topics(self):
+        return []
+
+    @property
+    def conductor_endpoints(self):
+        return []
+
+    @property
+    def notification_endpoints(self):
+        return []
