@@ -21,15 +21,15 @@ import copy
 import itertools
 import math
 import random
+import re
 
 import oslo_cache
 from oslo_config import cfg
 from oslo_log import log
+import oslo_utils
 
-from watcher._i18n import _LI, _
+from watcher._i18n import _
 from watcher.common import exception
-from watcher.decision_engine.cluster.history import ceilometer as \
-    ceilometer_cluster_history
 from watcher.decision_engine.model import element
 from watcher.decision_engine.strategy.strategies import base
 
@@ -49,42 +49,37 @@ def _set_memoize(conf):
 class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
     """Workload Stabilization control using live migration
 
-    *Description*
-
     This is workload stabilization strategy based on standard deviation
     algorithm. The goal is to determine if there is an overload in a cluster
     and respond to it by migrating VMs to stabilize the cluster.
 
-    *Requirements*
+    This strategy has been tested in a small (32 nodes) cluster.
 
-    * Software: Ceilometer component ceilometer-compute running
-      in each compute host, and Ceilometer API can report such telemetries
-      ``memory.resident`` and ``cpu_util`` successfully.
-    * You must have at least 2 physical compute nodes to run this strategy.
-
-    *Limitations*
-
-    - It assume that live migrations are possible
-    - Load on the system is sufficiently stable.
-
-    *Spec URL*
-
-    https://review.openstack.org/#/c/286153/
+    It assumes that live migrations are possible in your cluster.
     """
 
     MIGRATION = "migrate"
     MEMOIZE = _set_memoize(CONF)
 
+    DATASOURCE_METRICS = ['host_cpu_usage', 'instance_cpu_usage',
+                          'instance_ram_usage', 'host_memory_usage']
+
     def __init__(self, config, osc=None):
+        """Workload Stabilization control using live migration
+
+        :param config: A mapping containing the configuration of this strategy
+        :type config: :py:class:`~.Struct` instance
+        :param osc: :py:class:`~.OpenStackClients` instance
+        """
         super(WorkloadStabilization, self).__init__(config, osc)
-        self._ceilometer = None
-        self._nova = None
         self.weights = None
         self.metrics = None
         self.thresholds = None
         self.host_choice = None
         self.instance_metrics = None
         self.retry_count = None
+        self.periods = None
+        self.aggregation_method = None
 
     @classmethod
     def get_name(cls):
@@ -98,6 +93,10 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
     def get_translatable_display_name(cls):
         return "Workload stabilization"
 
+    @property
+    def granularity(self):
+        return self.input_parameters.get('granularity', 300)
+
     @classmethod
     def get_schema(cls):
         return {
@@ -105,19 +104,47 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
                 "metrics": {
                     "description": "Metrics used as rates of cluster loads.",
                     "type": "array",
-                    "default": ["cpu_util", "memory.resident"]
+                    "items": {
+                        "type": "string",
+                        "enum": ["cpu_util", "memory.resident"]
+                    },
+                    "default": ["cpu_util"]
                 },
                 "thresholds": {
                     "description": "Dict where key is a metric and value "
                                    "is a trigger value.",
                     "type": "object",
-                    "default": {"cpu_util": 0.2, "memory.resident": 0.2}
+                    "properties": {
+                        "cpu_util": {
+                            "type": "number",
+                            "minimum": 0,
+                            "maximum": 1
+                        },
+                        "memory.resident": {
+                            "type": "number",
+                            "minimum": 0,
+                            "maximum": 1
+                        }
+                    },
+                    "default": {"cpu_util": 0.1, "memory.resident": 0.1}
                 },
                 "weights": {
                     "description": "These weights used to calculate "
                                    "common standard deviation. Name of weight"
                                    " contains meter name and _weight suffix.",
                     "type": "object",
+                    "properties": {
+                        "cpu_util_weight": {
+                            "type": "number",
+                            "minimum": 0,
+                            "maximum": 1
+                        },
+                        "memory.resident_weight": {
+                            "type": "number",
+                            "minimum": 0,
+                            "maximum": 1
+                        }
+                    },
                     "default": {"cpu_util_weight": 1.0,
                                 "memory.resident_weight": 1.0}
                 },
@@ -125,7 +152,7 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
                     "description": "Mapping to get hardware statistics using"
                                    " instance metrics",
                     "type": "object",
-                    "default": {"cpu_util": "hardware.cpu.util",
+                    "default": {"cpu_util": "compute.node.cpu.percent",
                                 "memory.resident": "hardware.memory.used"}
                 },
                 "host_choice": {
@@ -142,31 +169,72 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
                 "retry_count": {
                     "description": "Count of random returned hosts",
                     "type": "number",
+                    "minimum": 1,
                     "default": 1
-                }
+                },
+                "periods": {
+                    "description": "These periods are used to get statistic "
+                                   "aggregation for instance and host "
+                                   "metrics. The period is simply a repeating"
+                                   " interval of time into which the samples"
+                                   " are grouped for aggregation. Watcher "
+                                   "uses only the last period of all received"
+                                   " ones.",
+                    "type": "object",
+                    "properties": {
+                        "instance": {
+                            "type": "integer",
+                            "minimum": 0
+                        },
+                        "node": {
+                            "type": "integer",
+                            "minimum": 0
+                        },
+                    },
+                    "default": {"instance": 720, "node": 600}
+                },
+                "aggregation_method": {
+                    "description": "Function used to aggregate multiple "
+                                   "measures into an aggregate. For example, "
+                                   "the min aggregation method will aggregate "
+                                   "the values of different measures to the "
+                                   "minimum value of all the measures in the "
+                                   "time range.",
+                    "type": "object",
+                    "properties": {
+                        "instance": {
+                            "type": "string",
+                            "default": 'mean'
+                        },
+                        "node": {
+                            "type": "string",
+                            "default": 'mean'
+                        },
+                    },
+                    "default": {"instance": 'mean', "node": 'mean'}
+                },
+                "granularity": {
+                    "description": "The time between two measures in an "
+                                   "aggregated timeseries of a metric.",
+                    "type": "number",
+                    "minimum": 0,
+                    "default": 300
+                },
             }
         }
 
-    @property
-    def ceilometer(self):
-        if self._ceilometer is None:
-            self._ceilometer = (ceilometer_cluster_history.
-                                CeilometerClusterHistory(osc=self.osc))
-        return self._ceilometer
-
-    @property
-    def nova(self):
-        if self._nova is None:
-            self._nova = self.osc.nova()
-        return self._nova
-
-    @nova.setter
-    def nova(self, n):
-        self._nova = n
-
-    @ceilometer.setter
-    def ceilometer(self, c):
-        self._ceilometer = c
+    @classmethod
+    def get_config_opts(cls):
+        return [
+            cfg.ListOpt(
+                "datasources",
+                help="Datasources to use in order to query the needed metrics."
+                     " If one of strategy metric isn't available in the first"
+                     " datasource, the next datasource will be chosen.",
+                item_type=cfg.types.String(choices=['gnocchi', 'ceilometer',
+                                                    'monasca']),
+                default=['gnocchi', 'ceilometer', 'monasca'])
+        ]
 
     def transform_instance_cpu(self, instance_load, host_vcpus):
         """Transform instance cpu utilization to overall host cpu utilization.
@@ -180,27 +248,27 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
                 (instance_load['vcpus'] / float(host_vcpus)))
 
     @MEMOIZE
-    def get_instance_load(self, instance_uuid):
-        """Gathering instance load through ceilometer statistic.
+    def get_instance_load(self, instance):
+        """Gathering instance load through ceilometer/gnocchi statistic.
 
-        :param instance_uuid: instance for which statistic is gathered.
+        :param instance: instance for which statistic is gathered.
         :return: dict
         """
         LOG.debug('get_instance_load started')
-        instance_vcpus = self.compute_model.get_resource_by_uuid(
-            element.ResourceType.cpu_cores).get_capacity(
-                self.compute_model.get_instance_by_uuid(instance_uuid))
-        instance_load = {'uuid': instance_uuid, 'vcpus': instance_vcpus}
+        instance_load = {'uuid': instance.uuid, 'vcpus': instance.vcpus}
         for meter in self.metrics:
-            avg_meter = self.ceilometer.statistic_aggregation(
-                resource_id=instance_uuid,
-                meter_name=meter,
-                period="120",
-                aggregate='min'
-            )
+            avg_meter = self.datasource_backend.statistic_aggregation(
+                instance.uuid, meter, self.periods['instance'],
+                self.granularity,
+                aggregation=self.aggregation_method['instance'])
             if avg_meter is None:
-                raise exception.NoMetricValuesForInstance(
-                    resource_id=instance_uuid, metric_name=meter)
+                LOG.warning(
+                    "No values returned by %(resource_id)s "
+                    "for %(metric_name)s", dict(
+                        resource_id=instance.uuid, metric_name=meter))
+                return
+            if meter == 'cpu_util':
+                avg_meter /= float(100)
             instance_load[meter] = avg_meter
         return instance_load
 
@@ -208,34 +276,45 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
         normalized_hosts = copy.deepcopy(hosts)
         for host in normalized_hosts:
             if 'memory.resident' in normalized_hosts[host]:
-                h_memory = self.compute_model.get_resource_by_uuid(
-                    element.ResourceType.memory).get_capacity(
-                        self.compute_model.get_node_by_uuid(host))
-                normalized_hosts[host]['memory.resident'] /= float(h_memory)
+                node = self.compute_model.get_node_by_uuid(host)
+                normalized_hosts[host]['memory.resident'] /= float(node.memory)
 
         return normalized_hosts
 
-    def get_hosts_load(self):
-        """Get load of every host by gathering instances load"""
-        hosts_load = {}
-        for node_id in self.compute_model.get_all_compute_nodes():
-            hosts_load[node_id] = {}
-            host_vcpus = self.compute_model.get_resource_by_uuid(
-                element.ResourceType.cpu_cores).get_capacity(
-                    self.compute_model.get_node_by_uuid(node_id))
-            hosts_load[node_id]['vcpus'] = host_vcpus
+    def get_available_nodes(self):
+        return {node_uuid: node for node_uuid, node in
+                self.compute_model.get_all_compute_nodes().items()
+                if node.state == element.ServiceState.ONLINE.value and
+                node.status == element.ServiceState.ENABLED.value}
 
+    def get_hosts_load(self):
+        """Get load of every available host by gathering instances load"""
+        hosts_load = {}
+        for node_id, node in self.get_available_nodes().items():
+            hosts_load[node_id] = {}
+            hosts_load[node_id]['vcpus'] = node.vcpus
             for metric in self.metrics:
-                avg_meter = self.ceilometer.statistic_aggregation(
-                    resource_id=node_id,
-                    meter_name=self.instance_metrics[metric],
-                    period="60",
-                    aggregate='avg'
-                )
+                resource_id = ''
+                avg_meter = None
+                meter_name = self.instance_metrics[metric]
+                if re.match('^compute.node', meter_name) is not None:
+                    resource_id = "%s_%s" % (node.uuid, node.hostname)
+                else:
+                    resource_id = node_id
+                avg_meter = self.datasource_backend.statistic_aggregation(
+                    resource_id, self.instance_metrics[metric],
+                    self.periods['node'], self.granularity,
+                    aggregation=self.aggregation_method['node'])
                 if avg_meter is None:
-                    raise exception.NoSuchMetricForHost(
-                        metric=self.instance_metrics[metric],
-                        host=node_id)
+                    LOG.warning('No values returned by node %s for %s',
+                                node_id, meter_name)
+                    del hosts_load[node_id]
+                    break
+                else:
+                    if meter_name == 'hardware.memory.used':
+                        avg_meter /= oslo_utils.units.Ki
+                    if meter_name == 'compute.node.cpu.percent':
+                        avg_meter /= 100
                 hosts_load[node_id][metric] = avg_meter
         return hosts_load
 
@@ -265,40 +344,49 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
                       " for %s in weight dict.") % metric)
         return weighted_sd
 
-    def calculate_migration_case(self, hosts, instance_id,
-                                 src_node_id, dst_node_id):
+    def calculate_migration_case(self, hosts, instance, src_node, dst_node):
         """Calculate migration case
 
         Return list of standard deviation values, that appearing in case of
         migration of instance from source host to destination host
         :param hosts: hosts with their workload
-        :param instance_id: the virtual machine
-        :param src_node_id: the source node id
-        :param dst_node_id: the destination node id
+        :param instance: the virtual machine
+        :param src_node: the source node
+        :param dst_node: the destination node
         :return: list of standard deviation values
         """
         migration_case = []
         new_hosts = copy.deepcopy(hosts)
-        instance_load = self.get_instance_load(instance_id)
-        d_host_vcpus = new_hosts[dst_node_id]['vcpus']
-        s_host_vcpus = new_hosts[src_node_id]['vcpus']
+        instance_load = self.get_instance_load(instance)
+        if not instance_load:
+            return
+        s_host_vcpus = new_hosts[src_node.uuid]['vcpus']
+        d_host_vcpus = new_hosts[dst_node.uuid]['vcpus']
         for metric in self.metrics:
-            if metric is 'cpu_util':
-                new_hosts[src_node_id][metric] -= self.transform_instance_cpu(
-                    instance_load,
-                    s_host_vcpus)
-                new_hosts[dst_node_id][metric] += self.transform_instance_cpu(
-                    instance_load,
-                    d_host_vcpus)
+            if metric == 'cpu_util':
+                new_hosts[src_node.uuid][metric] -= (
+                    self.transform_instance_cpu(instance_load, s_host_vcpus))
+                new_hosts[dst_node.uuid][metric] += (
+                    self.transform_instance_cpu(instance_load, d_host_vcpus))
             else:
-                new_hosts[src_node_id][metric] -= instance_load[metric]
-                new_hosts[dst_node_id][metric] += instance_load[metric]
+                new_hosts[src_node.uuid][metric] -= instance_load[metric]
+                new_hosts[dst_node.uuid][metric] += instance_load[metric]
         normalized_hosts = self.normalize_hosts_load(new_hosts)
         for metric in self.metrics:
             migration_case.append(self.get_sd(normalized_hosts, metric))
         migration_case.append(new_hosts)
         return migration_case
 
+    def get_current_weighted_sd(self, hosts_load):
+        """Calculate current weighted sd"""
+        current_sd = []
+        normalized_load = self.normalize_hosts_load(hosts_load)
+        for metric in self.metrics:
+            metric_sd = self.get_sd(normalized_load, metric)
+            current_sd.append(metric_sd)
+        current_sd.append(hosts_load)
+        return self.calculate_weighted_sd(current_sd[:-1])
+
     def simulate_migrations(self, hosts):
         """Make sorted list of pairs instance:dst_host"""
         def yield_nodes(nodes):
@@ -313,31 +401,39 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
                     yield nodes
 
         instance_host_map = []
-        nodes = list(self.compute_model.get_all_compute_nodes())
-        for source_hp_id in nodes:
+        nodes = sorted(list(self.get_available_nodes()))
+        current_weighted_sd = self.get_current_weighted_sd(hosts)
+        for src_host in nodes:
+            src_node = self.compute_model.get_node_by_uuid(src_host)
             c_nodes = copy.copy(nodes)
-            c_nodes.remove(source_hp_id)
+            c_nodes.remove(src_host)
             node_list = yield_nodes(c_nodes)
-            instances_id = self.compute_model.get_mapping(). \
-                get_node_instances_by_uuid(source_hp_id)
-            for instance_id in instances_id:
-                min_sd_case = {'value': len(self.metrics)}
-                instance = self.compute_model.get_instance_by_uuid(instance_id)
+            for instance in self.compute_model.get_node_instances(src_node):
+                # NOTE: skip exclude instance when migrating
+                if instance.watcher_exclude:
+                    LOG.debug("Instance is excluded by scope, "
+                              "skipped: %s", instance.uuid)
+                    continue
                 if instance.state not in [element.InstanceState.ACTIVE.value,
                                           element.InstanceState.PAUSED.value]:
                     continue
-                for dst_node_id in next(node_list):
-                    sd_case = self.calculate_migration_case(hosts, instance_id,
-                                                            source_hp_id,
-                                                            dst_node_id)
+                min_sd_case = {'value': current_weighted_sd}
+                for dst_host in next(node_list):
+                    dst_node = self.compute_model.get_node_by_uuid(dst_host)
+                    sd_case = self.calculate_migration_case(
+                        hosts, instance, src_node, dst_node)
+                    if sd_case is None:
+                        break
 
                     weighted_sd = self.calculate_weighted_sd(sd_case[:-1])
 
                     if weighted_sd < min_sd_case['value']:
                         min_sd_case = {
-                            'host': dst_node_id, 'value': weighted_sd,
-                            's_host': source_hp_id, 'instance': instance_id}
+                            'host': dst_node.uuid, 'value': weighted_sd,
+                            's_host': src_node.uuid, 'instance': instance.uuid}
                         instance_host_map.append(min_sd_case)
+                if sd_case is None:
+                    continue
         return sorted(instance_host_map, key=lambda x: x['value'])
 
     def check_threshold(self):
@@ -346,7 +442,12 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
         normalized_load = self.normalize_hosts_load(hosts_load)
         for metric in self.metrics:
             metric_sd = self.get_sd(normalized_load, metric)
+            LOG.info("Standard deviation for %s is %s.",
+                     (metric, metric_sd))
             if metric_sd > float(self.thresholds[metric]):
+                LOG.info("Standard deviation of %s exceeds"
+                         " appropriate threshold %s.",
+                         (metric, metric_sd))
                 return self.simulate_migrations(hosts_load)
 
     def add_migration(self,
@@ -384,17 +485,22 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
         return self.solution
 
     def pre_execute(self):
-        LOG.info(_LI("Initializing Workload Stabilization"))
+        LOG.info("Initializing Workload Stabilization")
 
         if not self.compute_model:
             raise exception.ClusterStateNotDefined()
 
+        if self.compute_model.stale:
+            raise exception.ClusterStateStale()
+
         self.weights = self.input_parameters.weights
         self.metrics = self.input_parameters.metrics
         self.thresholds = self.input_parameters.thresholds
         self.host_choice = self.input_parameters.host_choice
         self.instance_metrics = self.input_parameters.instance_metrics
         self.retry_count = self.input_parameters.retry_count
+        self.periods = self.input_parameters.periods
+        self.aggregation_method = self.input_parameters.aggregation_method
 
     def do_execute(self):
         migration = self.check_threshold()
@@ -403,19 +509,16 @@ class WorkloadStabilization(base.WorkloadStabilizationBaseStrategy):
             min_sd = 1
             balanced = False
             for instance_host in migration:
-                dst_hp_disk = self.compute_model.get_resource_by_uuid(
-                    element.ResourceType.disk).get_capacity(
-                        self.compute_model.get_node_by_uuid(
-                            instance_host['host']))
-                instance_disk = self.compute_model.get_resource_by_uuid(
-                    element.ResourceType.disk).get_capacity(
-                        self.compute_model.get_instance_by_uuid(
-                            instance_host['instance']))
-                if instance_disk > dst_hp_disk:
+                instance = self.compute_model.get_instance_by_uuid(
+                    instance_host['instance'])
+                src_node = self.compute_model.get_node_by_uuid(
+                    instance_host['s_host'])
+                dst_node = self.compute_model.get_node_by_uuid(
+                    instance_host['host'])
+                if instance.disk > dst_node.disk:
                     continue
                 instance_load = self.calculate_migration_case(
-                    hosts_load, instance_host['instance'],
-                    instance_host['s_host'], instance_host['host'])
+                    hosts_load, instance, src_node, dst_node)
                 weighted_sd = self.calculate_weighted_sd(instance_load[:-1])
                 if weighted_sd < min_sd:
                     min_sd = weighted_sd
