@@ -11,6 +11,7 @@
 #    limitations under the License.
 
 import datetime
+import itertools
 import mock
 
 from oslo_config import cfg
@@ -20,6 +21,7 @@ from wsme import types as wtypes
 from watcher.api.controllers.v1 import action as api_action
 from watcher.common import utils
 from watcher.db import api as db_api
+from watcher import objects
 from watcher.tests.api import base as api_base
 from watcher.tests.api import utils as api_utils
 from watcher.tests import base
@@ -33,7 +35,7 @@ def post_get_test_action(**kw):
     del action['action_plan_id']
     action['action_plan_uuid'] = kw.get('action_plan_uuid',
                                         action_plan['uuid'])
-    action['next'] = None
+    action['parents'] = None
     return action
 
 
@@ -41,7 +43,7 @@ class TestActionObject(base.TestCase):
 
     def test_action_init(self):
         action_dict = api_utils.action_post_data(action_plan_id=None,
-                                                 next=None)
+                                                 parents=None)
         del action_dict['state']
         action = api_action.Action(**action_dict)
         self.assertEqual(wtypes.Unset, action.state)
@@ -51,7 +53,10 @@ class TestListAction(api_base.FunctionalTest):
 
     def setUp(self):
         super(TestListAction, self).setUp()
-        obj_utils.create_test_action_plan(self.context)
+        self.goal = obj_utils.create_test_goal(self.context)
+        self.strategy = obj_utils.create_test_strategy(self.context)
+        self.audit = obj_utils.create_test_audit(self.context)
+        self.action_plan = obj_utils.create_test_action_plan(self.context)
 
     def test_empty(self):
         response = self.get_json('/actions')
@@ -63,13 +68,13 @@ class TestListAction(api_base.FunctionalTest):
             self.assertIn(field, action)
 
     def test_one(self):
-        action = obj_utils.create_test_action(self.context, next=None)
+        action = obj_utils.create_test_action(self.context, parents=None)
         response = self.get_json('/actions')
         self.assertEqual(action.uuid, response['actions'][0]["uuid"])
         self._assert_action_fields(response['actions'][0])
 
     def test_one_soft_deleted(self):
-        action = obj_utils.create_test_action(self.context, next=None)
+        action = obj_utils.create_test_action(self.context, parents=None)
         action.soft_delete()
         response = self.get_json('/actions',
                                  headers={'X-Show-Deleted': 'True'})
@@ -80,7 +85,7 @@ class TestListAction(api_base.FunctionalTest):
         self.assertEqual([], response['actions'])
 
     def test_get_one(self):
-        action = obj_utils.create_test_action(self.context, next=None)
+        action = obj_utils.create_test_action(self.context, parents=None)
         response = self.get_json('/actions/%s' % action['uuid'])
         self.assertEqual(action.uuid, response['uuid'])
         self.assertEqual(action.action_type, response['action_type'])
@@ -88,7 +93,7 @@ class TestListAction(api_base.FunctionalTest):
         self._assert_action_fields(response)
 
     def test_get_one_soft_deleted(self):
-        action = obj_utils.create_test_action(self.context, next=None)
+        action = obj_utils.create_test_action(self.context, parents=None)
         action.soft_delete()
         response = self.get_json('/actions/%s' % action['uuid'],
                                  headers={'X-Show-Deleted': 'True'})
@@ -100,13 +105,13 @@ class TestListAction(api_base.FunctionalTest):
         self.assertEqual(404, response.status_int)
 
     def test_detail(self):
-        action = obj_utils.create_test_action(self.context, next=None)
+        action = obj_utils.create_test_action(self.context, parents=None)
         response = self.get_json('/actions/detail')
         self.assertEqual(action.uuid, response['actions'][0]["uuid"])
         self._assert_action_fields(response['actions'][0])
 
     def test_detail_soft_deleted(self):
-        action = obj_utils.create_test_action(self.context, next=None)
+        action = obj_utils.create_test_action(self.context, parents=None)
         action.soft_delete()
         response = self.get_json('/actions/detail',
                                  headers={'X-Show-Deleted': 'True'})
@@ -117,7 +122,7 @@ class TestListAction(api_base.FunctionalTest):
         self.assertEqual([], response['actions'])
 
     def test_detail_against_single(self):
-        action = obj_utils.create_test_action(self.context, next=None)
+        action = obj_utils.create_test_action(self.context, parents=None)
         response = self.get_json('/actions/%s/detail' % action['uuid'],
                                  expect_errors=True)
         self.assertEqual(404, response.status_int)
@@ -152,12 +157,9 @@ class TestListAction(api_base.FunctionalTest):
             self.assertEqual(action_plan.uuid, action['action_plan_uuid'])
 
     def test_filter_by_audit_uuid(self):
-        audit = obj_utils.create_test_audit(self.context,
-                                            uuid=utils.generate_uuid())
         action_plan_1 = obj_utils.create_test_action_plan(
             self.context,
-            uuid=utils.generate_uuid(),
-            audit_id=audit.id)
+            uuid=utils.generate_uuid())
         action_list = []
 
         for id_ in range(3):
@@ -167,8 +169,9 @@ class TestListAction(api_base.FunctionalTest):
                 uuid=utils.generate_uuid())
             action_list.append(action.uuid)
 
-        audit2 = obj_utils.create_test_audit(self.context,
-                                             uuid=utils.generate_uuid())
+        audit2 = obj_utils.create_test_audit(
+            self.context, id=2, uuid=utils.generate_uuid(),
+            name='My Audit {0}'.format(2))
         action_plan_2 = obj_utils.create_test_action_plan(
             self.context,
             uuid=utils.generate_uuid(),
@@ -180,18 +183,16 @@ class TestListAction(api_base.FunctionalTest):
                 action_plan_id=action_plan_2.id,
                 uuid=utils.generate_uuid())
 
-        response = self.get_json('/actions?audit_uuid=%s' % audit.uuid)
+        response = self.get_json('/actions?audit_uuid=%s' % self.audit.uuid)
         self.assertEqual(len(action_list), len(response['actions']))
         for action in response['actions']:
             self.assertEqual(action_plan_1.uuid, action['action_plan_uuid'])
 
     def test_filter_by_action_plan_uuid(self):
-        audit = obj_utils.create_test_audit(self.context,
-                                            uuid=utils.generate_uuid())
         action_plan_1 = obj_utils.create_test_action_plan(
             self.context,
             uuid=utils.generate_uuid(),
-            audit_id=audit.id)
+            audit_id=self.audit.id)
         action_list = []
 
         for id_ in range(3):
@@ -204,7 +205,7 @@ class TestListAction(api_base.FunctionalTest):
         action_plan_2 = obj_utils.create_test_action_plan(
             self.context,
             uuid=utils.generate_uuid(),
-            audit_id=audit.id)
+            audit_id=self.audit.id)
 
         for id_ in range(4, 5, 6):
             obj_utils.create_test_action(
@@ -224,14 +225,12 @@ class TestListAction(api_base.FunctionalTest):
             self.assertEqual(action_plan_2.uuid, action['action_plan_uuid'])
 
     def test_details_and_filter_by_action_plan_uuid(self):
-        audit = obj_utils.create_test_audit(self.context,
-                                            uuid=utils.generate_uuid())
         action_plan = obj_utils.create_test_action_plan(
             self.context,
             uuid=utils.generate_uuid(),
-            audit_id=audit.id)
+            audit_id=self.audit.id)
 
-        for id_ in range(3):
+        for id_ in range(1, 3):
             action = obj_utils.create_test_action(
                 self.context, id=id_,
                 action_plan_id=action_plan.id,
@@ -243,36 +242,93 @@ class TestListAction(api_base.FunctionalTest):
             self.assertEqual(action_plan.uuid, action['action_plan_uuid'])
 
     def test_details_and_filter_by_audit_uuid(self):
-        audit = obj_utils.create_test_audit(self.context,
-                                            uuid=utils.generate_uuid())
         action_plan = obj_utils.create_test_action_plan(
             self.context,
             uuid=utils.generate_uuid(),
-            audit_id=audit.id)
+            audit_id=self.audit.id)
 
-        for id_ in range(3):
+        for id_ in range(1, 3):
             action = obj_utils.create_test_action(
                 self.context, id=id_,
                 action_plan_id=action_plan.id,
                 uuid=utils.generate_uuid())
 
         response = self.get_json(
-            '/actions/detail?audit_uuid=%s' % audit.uuid)
+            '/actions/detail?audit_uuid=%s' % self.audit.uuid)
         for action in response['actions']:
             self.assertEqual(action_plan.uuid, action['action_plan_uuid'])
 
     def test_filter_by_action_plan_and_audit_uuids(self):
-        audit = obj_utils.create_test_audit(
-            self.context, uuid=utils.generate_uuid())
         action_plan = obj_utils.create_test_action_plan(
             self.context,
             uuid=utils.generate_uuid(),
-            audit_id=audit.id)
+            audit_id=self.audit.id)
         url = '/actions?action_plan_uuid=%s&audit_uuid=%s' % (
-            action_plan.uuid, audit.uuid)
+            action_plan.uuid, self.audit.uuid)
         response = self.get_json(url, expect_errors=True)
         self.assertEqual(400, response.status_int)
 
+    def test_many_with_sort_key_uuid(self):
+        action_plan = obj_utils.create_test_action_plan(
+            self.context,
+            uuid=utils.generate_uuid(),
+            audit_id=self.audit.id)
+
+        actions_list = []
+        for id_ in range(1, 3):
+            action = obj_utils.create_test_action(
+                self.context, id=id_,
+                action_plan_id=action_plan.id,
+                uuid=utils.generate_uuid())
+            actions_list.append(action)
+
+        response = self.get_json('/actions?sort_key=%s' % 'uuid')
+        names = [s['uuid'] for s in response['actions']]
+
+        self.assertEqual(
+            sorted([a.uuid for a in actions_list]),
+            names)
+
+    def test_many_with_sort_key_action_plan_uuid(self):
+        action_plan_1 = obj_utils.create_test_action_plan(
+            self.context,
+            uuid=utils.generate_uuid(),
+            audit_id=self.audit.id)
+
+        action_plan_2 = obj_utils.create_test_action_plan(
+            self.context,
+            uuid=utils.generate_uuid(),
+            audit_id=self.audit.id)
+
+        action_plans_uuid_list = []
+        for id_, action_plan_id in enumerate(itertools.chain.from_iterable([
+                itertools.repeat(action_plan_1.id, 3),
+                itertools.repeat(action_plan_2.id, 2)]), 1):
+            action = obj_utils.create_test_action(
+                self.context, id=id_,
+                action_plan_id=action_plan_id,
+                uuid=utils.generate_uuid())
+            action_plans_uuid_list.append(action.action_plan.uuid)
+
+        for direction in ['asc', 'desc']:
+            response = self.get_json(
+                '/actions?sort_key={0}&sort_dir={1}'
+                .format('action_plan_uuid', direction))
+
+            action_plan_uuids = \
+                [s['action_plan_uuid'] for s in response['actions']]
+
+            self.assertEqual(
+                sorted(action_plans_uuid_list, reverse=(direction == 'desc')),
+                action_plan_uuids,
+                message='Failed on %s direction' % direction)
+
+    def test_sort_key_validation(self):
+        response = self.get_json(
+            '/actions?sort_key=%s' % 'bad_name',
+            expect_errors=True)
+        self.assertEqual(400, response.status_int)
+
     def test_many_with_soft_deleted_action_plan_uuid(self):
         action_plan1 = obj_utils.create_test_action_plan(
             self.context,
@@ -302,6 +358,8 @@ class TestListAction(api_base.FunctionalTest):
                 uuid=utils.generate_uuid())
             ap2_action_list.append(action)
 
+        action_plan1.state = objects.action_plan.State.CANCELLED
+        action_plan1.save()
         self.delete('/action_plans/%s' % action_plan1.uuid)
 
         response = self.get_json('/actions')
@@ -319,18 +377,23 @@ class TestListAction(api_base.FunctionalTest):
             set([act['uuid'] for act in response['actions']
                  if act['action_plan_uuid'] == action_plan2.uuid]))
 
-    def test_many_with_next_uuid(self):
+    def test_many_with_parents(self):
         action_list = []
         for id_ in range(5):
-            action = obj_utils.create_test_action(self.context, id=id_,
-                                                  uuid=utils.generate_uuid(),
-                                                  next=id_ + 1)
+            if id_ > 0:
+                action = obj_utils.create_test_action(
+                    self.context, id=id_, uuid=utils.generate_uuid(),
+                    parents=[action_list[id_ - 1]])
+            else:
+                action = obj_utils.create_test_action(
+                    self.context, id=id_, uuid=utils.generate_uuid(),
+                    parents=[])
             action_list.append(action.uuid)
         response = self.get_json('/actions')
         response_actions = response['actions']
-        for id in [0, 1, 2, 3]:
-            self.assertEqual(response_actions[id]['next_uuid'],
-                             response_actions[id + 1]['uuid'])
+        for id_ in range(4):
+            self.assertEqual(response_actions[id_]['uuid'],
+                             response_actions[id_ + 1]['parents'][0])
 
     def test_many_without_soft_deleted(self):
         action_list = []
@@ -364,30 +427,6 @@ class TestListAction(api_base.FunctionalTest):
         uuids = [s['uuid'] for s in response['actions']]
         self.assertEqual(sorted(action_list), sorted(uuids))
 
-    def test_many_with_sort_key_next_uuid(self):
-        for id_ in range(5):
-            obj_utils.create_test_action(self.context, id=id_,
-                                         uuid=utils.generate_uuid(),
-                                         next=id_ + 1)
-        response = self.get_json('/actions/')
-        reference_uuids = [
-            s.get('next_uuid', '') for s in response['actions']
-        ]
-
-        response = self.get_json('/actions/?sort_key=next_uuid')
-
-        self.assertEqual(5, len(response['actions']))
-        uuids = [(s['next_uuid'] if 'next_uuid' in s else '')
-                 for s in response['actions']]
-        self.assertEqual(sorted(reference_uuids), uuids)
-
-        response = self.get_json('/actions/?sort_key=next_uuid&sort_dir=desc')
-
-        self.assertEqual(5, len(response['actions']))
-        uuids = [(s['next_uuid'] if 'next_uuid' in s else '')
-                 for s in response['actions']]
-        self.assertEqual(sorted(reference_uuids, reverse=True), uuids)
-
     def test_links(self):
         uuid = utils.generate_uuid()
         obj_utils.create_test_action(self.context, id=1, uuid=uuid)
@@ -400,37 +439,33 @@ class TestListAction(api_base.FunctionalTest):
             self.assertTrue(self.validate_link(l['href'], bookmark=bookmark))
 
     def test_collection_links(self):
-        next = -1
+        parents = None
         for id_ in range(5):
             action = obj_utils.create_test_action(self.context, id=id_,
                                                   uuid=utils.generate_uuid(),
-                                                  next=next)
-            next = action.id
+                                                  parents=parents)
+            parents = [action.id]
         response = self.get_json('/actions/?limit=3')
         self.assertEqual(3, len(response['actions']))
 
-        next_marker = response['actions'][-1]['uuid']
-        self.assertIn(next_marker, response['next'])
-
     def test_collection_links_default_limit(self):
-        cfg.CONF.set_override('max_limit', 3, 'api',
-                              enforce_type=True)
+        cfg.CONF.set_override('max_limit', 3, 'api')
         for id_ in range(5):
             obj_utils.create_test_action(self.context, id=id_,
                                          uuid=utils.generate_uuid())
         response = self.get_json('/actions')
         self.assertEqual(3, len(response['actions']))
 
-        next_marker = response['actions'][-1]['uuid']
-        self.assertIn(next_marker, response['next'])
-
 
 class TestPatch(api_base.FunctionalTest):
 
     def setUp(self):
         super(TestPatch, self).setUp()
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
         obj_utils.create_test_action_plan(self.context)
-        self.action = obj_utils.create_test_action(self.context, next=None)
+        self.action = obj_utils.create_test_action(self.context, parents=None)
         p = mock.patch.object(db_api.BaseConnection, 'update_action')
         self.mock_action_update = p.start()
         self.mock_action_update.side_effect = self._simulate_rpc_action_update
@@ -444,15 +479,13 @@ class TestPatch(api_base.FunctionalTest):
     def test_patch_not_allowed(self, mock_utcnow):
         test_time = datetime.datetime(2000, 1, 1, 0, 0)
         mock_utcnow.return_value = test_time
-
-        new_state = 'SUBMITTED'
+        new_state = objects.audit.State.SUCCEEDED
         response = self.get_json('/actions/%s' % self.action.uuid)
         self.assertNotEqual(new_state, response['state'])
 
         response = self.patch_json(
             '/actions/%s' % self.action.uuid,
-            [{'path': '/state', 'value': new_state,
-             'op': 'replace'}],
+            [{'path': '/state', 'value': new_state, 'op': 'replace'}],
             expect_errors=True)
         self.assertEqual('application/json', response.content_type)
         self.assertEqual(403, response.status_int)
@@ -463,8 +496,11 @@ class TestDelete(api_base.FunctionalTest):
 
     def setUp(self):
         super(TestDelete, self).setUp()
-        obj_utils.create_test_action_plan(self.context)
-        self.action = obj_utils.create_test_action(self.context, next=None)
+        self.goal = obj_utils.create_test_goal(self.context)
+        self.strategy = obj_utils.create_test_strategy(self.context)
+        self.audit = obj_utils.create_test_audit(self.context)
+        self.action_plan = obj_utils.create_test_action_plan(self.context)
+        self.action = obj_utils.create_test_action(self.context, parents=None)
         p = mock.patch.object(db_api.BaseConnection, 'update_action')
         self.mock_action_update = p.start()
         self.mock_action_update.side_effect = self._simulate_rpc_action_update
@@ -487,6 +523,13 @@ class TestDelete(api_base.FunctionalTest):
 
 class TestActionPolicyEnforcement(api_base.FunctionalTest):
 
+    def setUp(self):
+        super(TestActionPolicyEnforcement, self).setUp()
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
+        obj_utils.create_test_action_plan(self.context)
+
     def _common_policy_check(self, rule, func, *arg, **kwarg):
         self.policy.set_rules({
             "admin_api": "(role:admin or role:administrator)",
