@@ -27,6 +27,12 @@ from watcher.tests.objects import utils as obj_utils
 
 class TestListActionPlan(api_base.FunctionalTest):
 
+    def setUp(self):
+        super(TestListActionPlan, self).setUp()
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
+
     def test_empty(self):
         response = self.get_json('/action_plans')
         self.assertEqual([], response['action_plans'])
@@ -71,14 +77,6 @@ class TestListActionPlan(api_base.FunctionalTest):
               'unit': '%'}],
             response['efficacy_indicators'])
 
-    def test_get_one_with_first_action(self):
-        action_plan = obj_utils.create_test_action_plan(self.context)
-        action = obj_utils.create_test_action(self.context, id=1)
-        response = self.get_json('/action_plans/%s' % action_plan['uuid'])
-        self.assertEqual(action_plan.uuid, response['uuid'])
-        self.assertEqual(action.uuid, response['first_action_uuid'])
-        self._assert_action_plans_fields(response)
-
     def test_get_one_soft_deleted(self):
         action_plan = obj_utils.create_test_action_plan(self.context)
         action_plan.soft_delete()
@@ -130,12 +128,12 @@ class TestListActionPlan(api_base.FunctionalTest):
 
     def test_many_with_soft_deleted_audit_uuid(self):
         action_plan_list = []
-        audit1 = obj_utils.create_test_audit(self.context,
-                                             id=1,
-                                             uuid=utils.generate_uuid())
-        audit2 = obj_utils.create_test_audit(self.context,
-                                             id=2,
-                                             uuid=utils.generate_uuid())
+        audit1 = obj_utils.create_test_audit(
+            self.context, id=2,
+            uuid=utils.generate_uuid(), name='My Audit {0}'.format(2))
+        audit2 = obj_utils.create_test_audit(
+            self.context, id=3,
+            uuid=utils.generate_uuid(), name='My Audit {0}'.format(3))
 
         for id_ in range(0, 2):
             action_plan = obj_utils.create_test_action_plan(
@@ -149,6 +147,11 @@ class TestListActionPlan(api_base.FunctionalTest):
                 audit_id=audit2.id)
             action_plan_list.append(action_plan.uuid)
 
+        new_state = objects.audit.State.CANCELLED
+        self.patch_json(
+            '/audits/%s' % audit1.uuid,
+            [{'path': '/state', 'value': new_state,
+             'op': 'replace'}])
         self.delete('/audits/%s' % audit1.uuid)
 
         response = self.get_json('/action_plans')
@@ -165,9 +168,10 @@ class TestListActionPlan(api_base.FunctionalTest):
 
     def test_many_with_audit_uuid(self):
         action_plan_list = []
-        audit = obj_utils.create_test_audit(self.context,
-                                            uuid=utils.generate_uuid())
-        for id_ in range(5):
+        audit = obj_utils.create_test_audit(
+            self.context, id=2,
+            uuid=utils.generate_uuid(), name='My Audit {0}'.format(2))
+        for id_ in range(2, 5):
             action_plan = obj_utils.create_test_action_plan(
                 self.context, id=id_, uuid=utils.generate_uuid(),
                 audit_id=audit.id)
@@ -179,16 +183,18 @@ class TestListActionPlan(api_base.FunctionalTest):
 
     def test_many_with_audit_uuid_filter(self):
         action_plan_list1 = []
-        audit1 = obj_utils.create_test_audit(self.context,
-                                             uuid=utils.generate_uuid())
-        for id_ in range(5):
+        audit1 = obj_utils.create_test_audit(
+            self.context, id=2,
+            uuid=utils.generate_uuid(), name='My Audit {0}'.format(2))
+        for id_ in range(2, 5):
             action_plan = obj_utils.create_test_action_plan(
                 self.context, id=id_, uuid=utils.generate_uuid(),
                 audit_id=audit1.id)
             action_plan_list1.append(action_plan.uuid)
 
-        audit2 = obj_utils.create_test_audit(self.context,
-                                             uuid=utils.generate_uuid())
+        audit2 = obj_utils.create_test_audit(
+            self.context, id=3,
+            uuid=utils.generate_uuid(), name='My Audit {0}'.format(3))
         action_plan_list2 = []
         for id_ in [5, 6, 7]:
             action_plan = obj_utils.create_test_action_plan(
@@ -235,9 +241,10 @@ class TestListActionPlan(api_base.FunctionalTest):
 
     def test_many_with_sort_key_audit_uuid(self):
         audit_list = []
-        for id_ in range(5):
-            audit = obj_utils.create_test_audit(self.context,
-                                                uuid=utils.generate_uuid())
+        for id_ in range(2, 5):
+            audit = obj_utils.create_test_audit(
+                self.context, id=id_,
+                uuid=utils.generate_uuid(), name='My Audit {0}'.format(id_))
             obj_utils.create_test_action_plan(
                 self.context, id=id_, uuid=utils.generate_uuid(),
                 audit_id=audit.id)
@@ -245,10 +252,16 @@ class TestListActionPlan(api_base.FunctionalTest):
 
         response = self.get_json('/action_plans/?sort_key=audit_uuid')
 
-        self.assertEqual(5, len(response['action_plans']))
+        self.assertEqual(3, len(response['action_plans']))
         uuids = [s['audit_uuid'] for s in response['action_plans']]
         self.assertEqual(sorted(audit_list), uuids)
 
+    def test_sort_key_validation(self):
+        response = self.get_json(
+            '/action_plans?sort_key=%s' % 'bad_name',
+            expect_errors=True)
+        self.assertEqual(400, response.status_int)
+
     def test_links(self):
         uuid = utils.generate_uuid()
         obj_utils.create_test_action_plan(self.context, id=1, uuid=uuid)
@@ -271,8 +284,7 @@ class TestListActionPlan(api_base.FunctionalTest):
         self.assertIn(next_marker, response['next'])
 
     def test_collection_links_default_limit(self):
-        cfg.CONF.set_override('max_limit', 3, 'api',
-                              enforce_type=True)
+        cfg.CONF.set_override('max_limit', 3, 'api')
         for id_ in range(5):
             obj_utils.create_test_action_plan(
                 self.context, id=id_, uuid=utils.generate_uuid())
@@ -287,6 +299,9 @@ class TestDelete(api_base.FunctionalTest):
 
     def setUp(self):
         super(TestDelete, self).setUp()
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
         self.action_plan = obj_utils.create_test_action_plan(
             self.context)
         p = mock.patch.object(db_api.BaseConnection, 'destroy_action_plan')
@@ -300,6 +315,13 @@ class TestDelete(api_base.FunctionalTest):
         action_plan.destroy()
 
     def test_delete_action_plan_without_action(self):
+        response = self.delete('/action_plans/%s' % self.action_plan.uuid,
+                               expect_errors=True)
+        self.assertEqual(400, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertTrue(response.json['error_message'])
+        self.action_plan.state = objects.action_plan.State.SUCCEEDED
+        self.action_plan.save()
         self.delete('/action_plans/%s' % self.action_plan.uuid)
         response = self.get_json('/action_plans/%s' % self.action_plan.uuid,
                                  expect_errors=True)
@@ -309,8 +331,10 @@ class TestDelete(api_base.FunctionalTest):
 
     def test_delete_action_plan_with_action(self):
         action = obj_utils.create_test_action(
-            self.context, id=self.action_plan.first_action_id)
+            self.context, id=1)
 
+        self.action_plan.state = objects.action_plan.State.SUCCEEDED
+        self.action_plan.save()
         self.delete('/action_plans/%s' % self.action_plan.uuid)
         ap_response = self.get_json('/action_plans/%s' % self.action_plan.uuid,
                                     expect_errors=True)
@@ -339,10 +363,60 @@ class TestDelete(api_base.FunctionalTest):
         self.assertTrue(response.json['error_message'])
 
 
+class TestStart(api_base.FunctionalTest):
+
+    def setUp(self):
+        super(TestStart, self).setUp()
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
+        self.action_plan = obj_utils.create_test_action_plan(
+            self.context, state=objects.action_plan.State.RECOMMENDED)
+        p = mock.patch.object(db_api.BaseConnection, 'update_action_plan')
+        self.mock_action_plan_update = p.start()
+        self.mock_action_plan_update.side_effect = \
+            self._simulate_rpc_action_plan_update
+        self.addCleanup(p.stop)
+
+    def _simulate_rpc_action_plan_update(self, action_plan):
+        action_plan.save()
+        return action_plan
+
+    @mock.patch('watcher.common.policy.enforce')
+    def test_start_action_plan_not_found(self, mock_policy):
+        mock_policy.return_value = True
+        uuid = utils.generate_uuid()
+        response = self.post('/v1/action_plans/%s/%s' %
+                             (uuid, 'start'), expect_errors=True)
+        self.assertEqual(404, response.status_int)
+        self.assertEqual('application/json', response.content_type)
+        self.assertTrue(response.json['error_message'])
+
+    @mock.patch('watcher.common.policy.enforce')
+    def test_start_action_plan(self, mock_policy):
+        mock_policy.return_value = True
+        action = obj_utils.create_test_action(
+            self.context, id=1)
+        self.action_plan.state = objects.action_plan.State.SUCCEEDED
+        response = self.post('/v1/action_plans/%s/%s/'
+                             % (self.action_plan.uuid, 'start'),
+                             expect_errors=True)
+        self.assertEqual(200, response.status_int)
+        act_response = self.get_json(
+            '/actions/%s' % action.uuid,
+            expect_errors=True)
+        self.assertEqual(200, act_response.status_int)
+        self.assertEqual('PENDING', act_response.json['state'])
+        self.assertEqual('application/json', act_response.content_type)
+
+
 class TestPatch(api_base.FunctionalTest):
 
     def setUp(self):
         super(TestPatch, self).setUp()
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
         self.action_plan = obj_utils.create_test_action_plan(
             self.context, state=objects.action_plan.State.RECOMMENDED)
         p = mock.patch.object(db_api.BaseConnection, 'update_action_plan')
@@ -360,7 +434,7 @@ class TestPatch(api_base.FunctionalTest):
         test_time = datetime.datetime(2000, 1, 1, 0, 0)
         mock_utcnow.return_value = test_time
 
-        new_state = 'DELETED'
+        new_state = objects.action_plan.State.DELETED
         response = self.get_json(
             '/action_plans/%s' % self.action_plan.uuid)
         self.assertNotEqual(new_state, response['state'])
@@ -449,7 +523,7 @@ ALLOWED_TRANSITIONS = [
     {"original_state": objects.action_plan.State.RECOMMENDED,
      "new_state": objects.action_plan.State.CANCELLED},
     {"original_state": objects.action_plan.State.ONGOING,
-     "new_state": objects.action_plan.State.CANCELLED},
+     "new_state": objects.action_plan.State.CANCELLING},
     {"original_state": objects.action_plan.State.PENDING,
      "new_state": objects.action_plan.State.CANCELLED},
 ]
@@ -482,6 +556,12 @@ class TestPatchStateTransitionDenied(api_base.FunctionalTest):
              "new_state": new_state} not in ALLOWED_TRANSITIONS
     ]
 
+    def setUp(self):
+        super(TestPatchStateTransitionDenied, self).setUp()
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
+
     @mock.patch.object(
         db_api.BaseConnection, 'update_action_plan',
         mock.Mock(side_effect=lambda ap: ap.save() or ap))
@@ -504,24 +584,6 @@ class TestPatchStateTransitionDenied(api_base.FunctionalTest):
         self.assertTrue(response.json['error_message'])
 
 
-class TestPatchStateDeletedNotFound(api_base.FunctionalTest):
-
-    @mock.patch.object(
-        db_api.BaseConnection, 'update_action_plan',
-        mock.Mock(side_effect=lambda ap: ap.save() or ap))
-    def test_replace_state_pending_not_found(self):
-        action_plan = obj_utils.create_test_action_plan(
-            self.context, state=objects.action_plan.State.DELETED)
-
-        response = self.get_json(
-            '/action_plans/%s' % action_plan.uuid,
-            expect_errors=True
-        )
-        self.assertEqual(404, response.status_code)
-        self.assertEqual('application/json', response.content_type)
-        self.assertTrue(response.json['error_message'])
-
-
 class TestPatchStateTransitionOk(api_base.FunctionalTest):
 
     scenarios = [
@@ -533,6 +595,12 @@ class TestPatchStateTransitionOk(api_base.FunctionalTest):
         for transition in ALLOWED_TRANSITIONS
     ]
 
+    def setUp(self):
+        super(TestPatchStateTransitionOk, self).setUp()
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
+
     @mock.patch.object(
         db_api.BaseConnection, 'update_action_plan',
         mock.Mock(side_effect=lambda ap: ap.save() or ap))
@@ -547,7 +615,6 @@ class TestPatchStateTransitionOk(api_base.FunctionalTest):
             '/action_plans/%s' % action_plan.uuid,
             [{'path': '/state', 'value': self.new_state, 'op': 'replace'}])
         updated_ap = self.get_json('/action_plans/%s' % action_plan.uuid)
-
         self.assertNotEqual(self.new_state, initial_ap['state'])
         self.assertEqual(self.new_state, updated_ap['state'])
         self.assertEqual('application/json', response.content_type)
@@ -556,6 +623,12 @@ class TestPatchStateTransitionOk(api_base.FunctionalTest):
 
 class TestActionPlanPolicyEnforcement(api_base.FunctionalTest):
 
+    def setUp(self):
+        super(TestActionPlanPolicyEnforcement, self).setUp()
+        obj_utils.create_test_goal(self.context)
+        obj_utils.create_test_strategy(self.context)
+        obj_utils.create_test_audit(self.context)
+
     def _common_policy_check(self, rule, func, *arg, **kwarg):
         self.policy.set_rules({
             "admin_api": "(role:admin or role:administrator)",
@@ -591,7 +664,9 @@ class TestActionPlanPolicyEnforcement(api_base.FunctionalTest):
         self._common_policy_check(
             "action_plan:update", self.patch_json,
             '/action_plans/%s' % action_plan.uuid,
-            [{'path': '/state', 'value': 'DELETED', 'op': 'replace'}],
+            [{'path': '/state',
+              'value': objects.action_plan.State.DELETED,
+              'op': 'replace'}],
             expect_errors=True)
 
     def test_policy_disallow_delete(self):
@@ -613,4 +688,5 @@ class TestActionPlanPolicyEnforcementWithAdminContext(TestListActionPlan,
             "action_plan:detail": "rule:default",
             "action_plan:get": "rule:default",
             "action_plan:get_all": "rule:default",
-            "action_plan:update": "rule:default"})
+            "action_plan:update": "rule:default",
+            "action_plan:start": "rule:default"})
