@@ -26,17 +26,16 @@ from watcher.common import exception
 from watcher.common import nova_helper
 from watcher.common import service as watcher_service
 from watcher.decision_engine.model import element
-from watcher.decision_engine.model import model_root
 from watcher.decision_engine.model.notification import nova as novanotification
 from watcher.tests import base as base_test
+from watcher.tests.decision_engine.model import faker_cluster_state
 from watcher.tests.decision_engine.model.notification import fake_managers
-from watcher.tests.decision_engine.strategy.strategies \
-    import faker_cluster_state
 
 
 class NotificationTestCase(base_test.TestCase):
 
-    def load_message(self, filename):
+    @staticmethod
+    def load_message(filename):
         cwd = os.path.abspath(os.path.dirname(__file__))
         data_folder = os.path.join(cwd, "data")
 
@@ -57,6 +56,10 @@ class TestReceiveNovaNotifications(NotificationTestCase):
         m_from_dict = p_from_dict.start()
         m_from_dict.return_value = self.context
         self.addCleanup(p_from_dict.stop)
+        p_heartbeat = mock.patch.object(
+            watcher_service.ServiceHeartbeat, "send_beat")
+        self.m_heartbeat = p_heartbeat.start()
+        self.addCleanup(p_heartbeat.stop)
 
     @mock.patch.object(novanotification.ServiceUpdated, 'info')
     def test_nova_receive_service_update(self, m_info):
@@ -128,7 +131,7 @@ class TestNovaNotifications(NotificationTestCase):
         node0_uuid = 'Node_0'
         node0 = compute_model.get_node_by_uuid(node0_uuid)
 
-        message = self.load_message('scenario3_service-update.json')
+        message = self.load_message('scenario3_service-update-disabled.json')
 
         self.assertEqual('hostname_0', node0.hostname)
         self.assertEqual(element.ServiceState.ONLINE.value, node0.state)
@@ -146,6 +149,20 @@ class TestNovaNotifications(NotificationTestCase):
         self.assertEqual(element.ServiceState.OFFLINE.value, node0.state)
         self.assertEqual(element.ServiceState.DISABLED.value, node0.status)
 
+        message = self.load_message('scenario3_service-update-enabled.json')
+
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+
+        self.assertEqual('Node_0', node0.hostname)
+        self.assertEqual(element.ServiceState.ONLINE.value, node0.state)
+        self.assertEqual(element.ServiceState.ENABLED.value, node0.status)
+
     def test_nova_instance_update(self):
         compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
         self.fake_cdmc.cluster_data_model = compute_model
@@ -175,6 +192,9 @@ class TestNovaNotifications(NotificationTestCase):
             side_effect=lambda uuid: mock.Mock(
                 name='m_get_compute_node_by_hostname',
                 id=3,
+                hypervisor_hostname="Node_2",
+                state='up',
+                status='enabled',
                 uuid=uuid,
                 memory_mb=7777,
                 vcpus=42,
@@ -183,7 +203,6 @@ class TestNovaNotifications(NotificationTestCase):
         m_nova_helper_cls.return_value = mock.Mock(
             get_compute_node_by_hostname=m_get_compute_node_by_hostname,
             name='m_nova_helper')
-
         compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
         self.fake_cdmc.cluster_data_model = compute_model
         handler = novanotification.InstanceUpdated(self.fake_cdmc)
@@ -201,32 +220,24 @@ class TestNovaNotifications(NotificationTestCase):
         )
 
         instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
-        cpu_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.cpu_cores)
-        disk = compute_model.get_resource_by_uuid(
-            element.ResourceType.disk)
-        disk_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.disk_capacity)
-        memory_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.memory)
 
         self.assertEqual(element.InstanceState.PAUSED.value, instance0.state)
-        self.assertEqual(1, cpu_capacity.get_capacity(instance0))
-        self.assertEqual(1, disk_capacity.get_capacity(instance0))
-        self.assertEqual(512, memory_capacity.get_capacity(instance0))
+        self.assertEqual(1, instance0.vcpus)
+        self.assertEqual(1, instance0.disk_capacity)
+        self.assertEqual(512, instance0.memory)
 
         m_get_compute_node_by_hostname.assert_called_once_with('Node_2')
         node_2 = compute_model.get_node_by_uuid('Node_2')
-        self.assertEqual(7777, memory_capacity.get_capacity(node_2))
-        self.assertEqual(42, cpu_capacity.get_capacity(node_2))
-        self.assertEqual(974, disk.get_capacity(node_2))
-        self.assertEqual(1337, disk_capacity.get_capacity(node_2))
+        self.assertEqual(7777, node_2.memory)
+        self.assertEqual(42, node_2.vcpus)
+        self.assertEqual(974, node_2.disk)
+        self.assertEqual(1337, node_2.disk_capacity)
 
     @mock.patch.object(nova_helper, "NovaHelper")
     def test_instance_update_node_notfound_set_unmapped(
             self, m_nova_helper_cls):
         m_get_compute_node_by_hostname = mock.Mock(
-            side_effect=exception.ComputeNodeNotFound)
+            side_effect=exception.ComputeNodeNotFound(name="TEST"))
         m_nova_helper_cls.return_value = mock.Mock(
             get_compute_node_by_hostname=m_get_compute_node_by_hostname,
             name='m_nova_helper')
@@ -249,20 +260,12 @@ class TestNovaNotifications(NotificationTestCase):
         )
 
         instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
-        cpu_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.cpu_cores)
-        disk = compute_model.get_resource_by_uuid(
-            element.ResourceType.disk)
-        disk_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.disk_capacity)
-        memory_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.memory)
 
         self.assertEqual(element.InstanceState.PAUSED.value, instance0.state)
-        self.assertEqual(1, cpu_capacity.get_capacity(instance0))
-        self.assertEqual(1, disk.get_capacity(instance0))
-        self.assertEqual(1, disk_capacity.get_capacity(instance0))
-        self.assertEqual(512, memory_capacity.get_capacity(instance0))
+        self.assertEqual(1, instance0.vcpus)
+        self.assertEqual(1, instance0.disk)
+        self.assertEqual(1, instance0.disk_capacity)
+        self.assertEqual(512, instance0.memory)
 
         m_get_compute_node_by_hostname.assert_any_call('Node_2')
         self.assertRaises(
@@ -290,17 +293,11 @@ class TestNovaNotifications(NotificationTestCase):
         )
 
         instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
-        cpu_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.cpu_cores)
-        disk_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.disk)
-        memory_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.memory)
 
         self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
-        self.assertEqual(1, cpu_capacity.get_capacity(instance0))
-        self.assertEqual(1, disk_capacity.get_capacity(instance0))
-        self.assertEqual(512, memory_capacity.get_capacity(instance0))
+        self.assertEqual(1, instance0.vcpus)
+        self.assertEqual(1, instance0.disk_capacity)
+        self.assertEqual(512, instance0.memory)
 
     def test_nova_instance_delete_end(self):
         compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
@@ -311,8 +308,6 @@ class TestNovaNotifications(NotificationTestCase):
 
         # Before
         self.assertTrue(compute_model.get_instance_by_uuid(instance0_uuid))
-        for resource in compute_model.resource.values():
-            self.assertIn(instance0_uuid, resource.mapping)
 
         message = self.load_message('scenario3_instance-delete-end.json')
         handler.info(
@@ -328,9 +323,6 @@ class TestNovaNotifications(NotificationTestCase):
             exception.InstanceNotFound,
             compute_model.get_instance_by_uuid, instance0_uuid)
 
-        for resource in compute_model.resource.values():
-            self.assertNotIn(instance0_uuid, resource.mapping)
-
 
 class TestLegacyNovaNotifications(NotificationTestCase):
 
@@ -363,17 +355,11 @@ class TestLegacyNovaNotifications(NotificationTestCase):
         )
 
         instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
-        cpu_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.cpu_cores)
-        disk_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.disk)
-        memory_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.memory)
 
         self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
-        self.assertEqual(1, cpu_capacity.get_capacity(instance0))
-        self.assertEqual(1, disk_capacity.get_capacity(instance0))
-        self.assertEqual(512, memory_capacity.get_capacity(instance0))
+        self.assertEqual(1, instance0.vcpus)
+        self.assertEqual(1, instance0.disk_capacity)
+        self.assertEqual(512, instance0.memory)
 
     def test_legacy_instance_updated(self):
         compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
@@ -397,31 +383,6 @@ class TestLegacyNovaNotifications(NotificationTestCase):
 
         self.assertEqual(element.InstanceState.PAUSED.value, instance0.state)
 
-    def test_legacy_instance_update_instance_notfound_creates(self):
-        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
-        self.fake_cdmc.cluster_data_model = compute_model
-        handler = novanotification.LegacyInstanceUpdated(self.fake_cdmc)
-
-        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
-
-        message = self.load_message('scenario3_legacy_instance-update.json')
-
-        with mock.patch.object(
-            model_root.ModelRoot, 'get_instance_by_uuid'
-        ) as m_get_instance_by_uuid:
-            m_get_instance_by_uuid.side_effect = exception.InstanceNotFound(
-                name='TEST')
-            handler.info(
-                ctxt=self.context,
-                publisher_id=message['publisher_id'],
-                event_type=message['event_type'],
-                payload=message['payload'],
-                metadata=self.FAKE_METADATA,
-            )
-
-        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
-        self.assertEqual(element.InstanceState.PAUSED.value, instance0.state)
-
     @mock.patch.object(nova_helper, "NovaHelper")
     def test_legacy_instance_update_node_notfound_still_creates(
             self, m_nova_helper_cls):
@@ -430,6 +391,9 @@ class TestLegacyNovaNotifications(NotificationTestCase):
                 name='m_get_compute_node_by_hostname',
                 id=3,
                 uuid=uuid,
+                hypervisor_hostname="Node_2",
+                state='up',
+                status='enabled',
                 memory_mb=7777,
                 vcpus=42,
                 free_disk_gb=974,
@@ -456,27 +420,19 @@ class TestLegacyNovaNotifications(NotificationTestCase):
         )
 
         instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
-        cpu_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.cpu_cores)
-        disk = compute_model.get_resource_by_uuid(
-            element.ResourceType.disk)
-        disk_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.disk_capacity)
-        memory_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.memory)
 
         self.assertEqual(element.InstanceState.PAUSED.value, instance0.state)
-        self.assertEqual(1, cpu_capacity.get_capacity(instance0))
-        self.assertEqual(1, disk.get_capacity(instance0))
-        self.assertEqual(1, disk_capacity.get_capacity(instance0))
-        self.assertEqual(512, memory_capacity.get_capacity(instance0))
+        self.assertEqual(1, instance0.vcpus)
+        self.assertEqual(1, instance0.disk)
+        self.assertEqual(1, instance0.disk_capacity)
+        self.assertEqual(512, instance0.memory)
 
         m_get_compute_node_by_hostname.assert_any_call('Node_2')
         node_2 = compute_model.get_node_by_uuid('Node_2')
-        self.assertEqual(7777, memory_capacity.get_capacity(node_2))
-        self.assertEqual(42, cpu_capacity.get_capacity(node_2))
-        self.assertEqual(974, disk.get_capacity(node_2))
-        self.assertEqual(1337, disk_capacity.get_capacity(node_2))
+        self.assertEqual(7777, node_2.memory)
+        self.assertEqual(42, node_2.vcpus)
+        self.assertEqual(974, node_2.disk)
+        self.assertEqual(1337, node_2.disk_capacity)
 
     @mock.patch.object(nova_helper, "NovaHelper")
     def test_legacy_instance_update_node_notfound_set_unmapped(
@@ -505,20 +461,12 @@ class TestLegacyNovaNotifications(NotificationTestCase):
         )
 
         instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
-        cpu_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.cpu_cores)
-        disk = compute_model.get_resource_by_uuid(
-            element.ResourceType.disk)
-        disk_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.disk_capacity)
-        memory_capacity = compute_model.get_resource_by_uuid(
-            element.ResourceType.memory)
 
         self.assertEqual(element.InstanceState.PAUSED.value, instance0.state)
-        self.assertEqual(1, cpu_capacity.get_capacity(instance0))
-        self.assertEqual(1, disk.get_capacity(instance0))
-        self.assertEqual(1, disk_capacity.get_capacity(instance0))
-        self.assertEqual(512, memory_capacity.get_capacity(instance0))
+        self.assertEqual(1, instance0.vcpus)
+        self.assertEqual(1, instance0.disk)
+        self.assertEqual(1, instance0.disk_capacity)
+        self.assertEqual(512, instance0.memory)
 
         m_get_compute_node_by_hostname.assert_any_call('Node_2')
         self.assertRaises(
@@ -558,8 +506,6 @@ class TestLegacyNovaNotifications(NotificationTestCase):
 
         # Before
         self.assertTrue(compute_model.get_instance_by_uuid(instance0_uuid))
-        for resource in compute_model.resource.values():
-            self.assertIn(instance0_uuid, resource.mapping)
 
         message = self.load_message(
             'scenario3_legacy_instance-delete-end.json')
@@ -576,5 +522,50 @@ class TestLegacyNovaNotifications(NotificationTestCase):
             exception.InstanceNotFound,
             compute_model.get_instance_by_uuid, instance0_uuid)
 
-        for resource in compute_model.resource.values():
-            self.assertNotIn(instance0_uuid, resource.mapping)
+    def test_legacy_instance_resize_confirm_end(self):
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.LegacyLiveMigratedEnd(self.fake_cdmc)
+
+        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
+
+        node = compute_model.get_node_by_instance_uuid(instance0_uuid)
+        self.assertEqual('Node_0', node.uuid)
+
+        message = self.load_message(
+            'scenario3_legacy_instance-resize-confirm-end.json')
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+        node = compute_model.get_node_by_instance_uuid(instance0_uuid)
+        self.assertEqual('Node_1', node.uuid)
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
+
+    def test_legacy_instance_rebuild_end(self):
+        compute_model = self.fake_cdmc.generate_scenario_3_with_2_nodes()
+        self.fake_cdmc.cluster_data_model = compute_model
+        handler = novanotification.LegacyLiveMigratedEnd(self.fake_cdmc)
+
+        instance0_uuid = '73b09e16-35b7-4922-804e-e8f5d9b740fc'
+        instance0 = compute_model.get_instance_by_uuid(instance0_uuid)
+
+        node = compute_model.get_node_by_instance_uuid(instance0_uuid)
+        self.assertEqual('Node_0', node.uuid)
+
+        message = self.load_message(
+            'scenario3_legacy_instance-rebuild-end.json')
+        handler.info(
+            ctxt=self.context,
+            publisher_id=message['publisher_id'],
+            event_type=message['event_type'],
+            payload=message['payload'],
+            metadata=self.FAKE_METADATA,
+        )
+        node = compute_model.get_node_by_instance_uuid(instance0_uuid)
+        self.assertEqual('Node_1', node.uuid)
+        self.assertEqual(element.InstanceState.ACTIVE.value, instance0.state)
